(function (Drupal) {
  Drupal.behaviors.vizWidgetMenuTreeV2 = {
    attach() {
      once('vizWidgetMenuTreeV2', '[data-widget="viz_widgets:menu-tree-v2"]').forEach(inintialMenuTree);
    },
  };
  /**
   * Initialize the menu tree
   * @param {HTMLElement} wrapper
   */
  function inintialMenuTree(wrapper) {
    let currentMode = 'desktop';

    const container = wrapper.querySelector('.menu-tree.menu-tree--default');
    const computedEntryHeight = getComputedStyle(container.querySelector('.menu-tree__menu-item__entry__depth-0')).height || '56px';
    const entryHeight = parseInt((computedEntryHeight.replace('px', '')), 10);
    const isAccordion
      = container.getAttribute('data-vizcms-accordion') === null
      || container.getAttribute('data-vizcms-accordion') === 'true';

    const menuContainer = container.querySelector('.menu-tree__menu--root');
    const entries = container.querySelectorAll('.menu-tree__menu-item__entry.has-children');
    const childMenus = container.querySelectorAll('.menu-tree__menu--children');
    const toggleMobile = container.querySelector('.menu-tree__mobile-toggle');

    if (window.innerWidth >= 1024) {
      currentMode = 'desktop';
      menuContainer.style.height = 'auto';
    } else {
      currentMode = 'mobile';
      hideMenuOnMobile();
    }

    window.addEventListener('resize', () => {
      if (window.innerWidth >= 1024 && currentMode === 'mobile') {
        currentMode = 'desktop';
        menuContainer.style.height = 'auto';
      } else if (window.innerWidth < 1024 && currentMode === 'desktop') {
        currentMode = 'mobile';
        hideMenuOnMobile();
      }
    });

    function hideMenuOnMobile() {
      menuContainer.style.height = '0px';
      menuContainer.classList.remove('hidden');
      hideSubmenus(menuContainer);
    }

    menuContainer.addEventListener('transitionend', () => {
      if (menuContainer.style.height === '0px') {
        hideMenuOnMobile();
      } else {
        menuContainer.style.height = 'auto';
      }
    });

    toggleMobile.addEventListener('click', () => {
      if (menuContainer.style.height === 'auto') {
        menuContainer.style.height = menuContainer.scrollHeight + 'px';
        setTimeout(() => {
          menuContainer.style.height = '0px';
        }, 50);
      } else {
        menuContainer.style.height = (menuContainer.children.length * entryHeight) + 'px';
      }
    });

    childMenus.forEach((menu) => {
      // Hide submenus when the menu close and set the height to auto when the menu open
      menu.addEventListener('transitionend', () => {
        if (menu.style.height === '0px') {
          hideSubmenus(menu);
        } else {
          menu.style.height = 'auto';
        }
      });
    });

    // For each entry click event
    entries.forEach((entry) => {
      entry.addEventListener('click', () => {
        // Accordion effect
        if (isAccordion) {
          hidenSiblingMenus(entry);
        }

        // Open/close menu
        const menu = entry.parentElement.querySelector('ul');
        if (menu.clientHeight > 0) {
          hideMenu(menu);
          entry.classList.remove('is-expanded');
        } else {
          menu.style.height = (menu.children.length * entryHeight) + 'px';
          entry.classList.add('is-expanded');
        }
      });
    });

    /**
     * Find and hide sibling menus
     * @param {HTMLElement} entry
     */
    function hidenSiblingMenus(entry) {
      const siblingCollection = entry.parentElement.parentElement.children;
      Array.from(siblingCollection).forEach((sibling) => {
        if (sibling !== entry.parentElement) {
          sibling.querySelector('.entry').classList.remove('is-expanded');
          const siblingMenu = sibling.querySelector('ul');
          if (siblingMenu && siblingMenu.style.height !== '0px') {
            hideMenu(siblingMenu);
          }
        }
      });
    }

    function hideSubmenus(menu) {
      menu.querySelectorAll('ul').forEach((submenu) => {
        submenu.style.height = '0px';
      });
      menu.querySelectorAll('.is-expanded').forEach((entry) => {
        entry.classList.remove('is-expanded');
      });
    }

    /**
     * Hide menu with transition
     * @param {HTMLElement} menu
     */
    function hideMenu(menu) {
      menu.style.height = menu.scrollHeight + 'px';
      // For transition effect to work, we need to wait a bit
      setTimeout(() => {
        menu.style.height = '0px';
      }, 50);
    }
  }
})(Drupal);
